@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2019, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -33,6 +33,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -55,10 +56,6 @@
 #define fts_set fts_set_compat
 #endif
 
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-
 #include "bookkeeper.h"
 #include "expire.h"
 #include "nfstatfile.h"
@@ -305,7 +302,7 @@ void RescanDir(char *dir, dirstat_t *dirstat) {
                 case FTS_D:
                     // skip all '.' entries as well as hidden directories
                     if (ftsent->fts_level > 0 && ftsent->fts_name[0] == '.') fts_set(fts, ftsent, FTS_SKIP);
-                    // any valid dirctory need to start with a digit ( %Y -> year )
+                    // any valid directory need to start with a digit ( %Y -> year )
                     if (ftsent->fts_level > 0 && !isdigit(ftsent->fts_name[0])) fts_set(fts, ftsent, FTS_SKIP);
                     break;
                 case FTS_DP:
@@ -430,7 +427,7 @@ void ExpireDir(char *dir, dirstat_t *dirstat, uint64_t maxsize, uint64_t maxlife
                 }
 
                 // expire time-wise if needed
-                // this part of the code is executed only when size-wise is fullfilled
+                // this part of the code is executed only when size-wise is fulfilled
                 if (!lifetime_done) {
                     if (expire_timelimit && strcmp(p, expire_timelimit) < 0) {
                         if (unlink(ftsent->fts_path) == 0) {
@@ -547,7 +544,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
     int size_done, lifetime_done, done;
     char *expire_timelimit = "";
     time_t now = time(NULL);
-    uint64_t sizelimit, num_expired;
+    uint64_t sizelimit;
 
     if (!channel) return;
 
@@ -567,8 +564,6 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
     sizelimit = (current_stat->low_water * maxsize) / 100;
     lifetime_done = maxlife == 0 || (now - current_stat->first) < maxlife;
 
-    num_expired = 0;
-
     PrepareDirLists(channel);
     if (runtime) alarm(runtime);
     while (!done) {
@@ -625,7 +620,6 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
                     expire_channel->ftsent->fts_number--;
 
                     file_removed = 1;
-                    num_expired++;
                 } else {
                     LogError("unlink() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
                 }
@@ -638,7 +632,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
         } else if (!lifetime_done) {
             dbg_printf("	Time expire \n");
             // expire time-wise if needed
-            // this part of the code is executed only when size-wise is already fullfilled
+            // this part of the code is executed only when size-wise is already fulfilled
             if (strcmp(p, expire_timelimit) < 0) {
                 // need to delete this file
                 if (unlink(expire_channel->ftsent->fts_path) == 0) {
@@ -654,7 +648,6 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
                     expire_channel->ftsent->fts_number--;
 
                     file_removed = 1;
-                    num_expired++;
                 } else {
                     LogError("unlink() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
                 }
@@ -718,7 +711,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
             expire_channel->dirstat->first = expire_channel->dirstat->last;
             if (expire_channel->dirstat->numfiles) {
                 // if channel is empty, no files must be reported, but rebuild is done anyway
-                LogError("Inconsitency detected in channel %s. Will rebuild automatically.\n", expire_channel->datadir);
+                LogError("Inconsistency detected in channel %s. Will rebuild automatically.\n", expire_channel->datadir);
                 LogError("No more files found, but %llu expected.\n", expire_channel->dirstat->numfiles);
             }
             expire_channel->dirstat->numfiles = 0;
@@ -735,7 +728,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
 
 void UpdateDirStat(dirstat_t *dirstat, bookkeeper_t *books) {
     if (books->numfiles) {
-        /* prevent some faults and dublicates:
+        /* prevent some faults and duplicates:
          * book records can never be timewise smaller than directory records => fishy!
          * in case book records == directory records, the user stopped and restarted nfcapd
          * this is not necessarily wrong, but results in overwriting an existing file
