@@ -33,6 +33,8 @@ static inline size_t CheckBufferSpace(nffile_t *nffile, size_t required);
 
 static inline void AppendToBuffer(nffile_t *nffile, void *record, size_t required);
 
+static inline void ClearMasterRecord(master_record_t *record);
+
 static inline void ExpandRecord_v3(recordHeaderV3_t *v3Record, master_record_t *output_record);
 
 #ifdef NEED_PACKRECORD
@@ -49,8 +51,8 @@ static inline size_t CheckBufferSpace(nffile_t *nffile, size_t required) {
 
     // flush current buffer to disc
     if ((nffile->block_header->size + required) > WRITE_BUFFSIZE) {
-        // this should never happen, but catch it anyway
         if (required > WRITE_BUFFSIZE) {
+            // this should never happen, but catch it anyway
             LogError("Required buffer size %zu too big for output buffer!", required);
             return 0;
         }
@@ -66,6 +68,12 @@ static inline size_t CheckBufferSpace(nffile_t *nffile, size_t required) {
 
 }  // End of CheckBufferSpace
 
+static inline void ClearMasterRecord(master_record_t *record) {
+    if (record->inPayload) free(record->inPayload);
+    if (record->outPayload) free(record->outPayload);
+    memset((void *)record, 0, sizeof(master_record_t));
+}  // End of ClearMasterRecord
+
 static inline void ExpandRecord_v3(recordHeaderV3_t *v3Record, master_record_t *output_record) {
     elementHeader_t *elementHeader;
     uint32_t size = sizeof(recordHeaderV3_t);
