@@ -31,6 +31,7 @@
 #ifndef _NFFILE_H
 #define _NFFILE_H 1
 
+#include <pthread.h>
 #include <stddef.h>
 #include <sys/types.h>
 
@@ -182,17 +183,21 @@ typedef struct data_block_header_s {
                           // 2 - block compressed
 } data_block_headerV1_t;
 
+// allocate space for this number of workers
+#define MAXWORKERS 16
+// If number of CPUs can not be determined
+#define DEFAULTWORKERS 4
 /*
  * Generic file handle for reading/writing files
  * if a file is read only writeto and block_header are NULL
  */
 typedef struct nffile_s {
-    fileHeaderV2_t *file_header;  // file header
-    int fd;                       // associated file descriptor
-    int compat16;                 // underlying file is compat16
-    pthread_t worker;             // nfread/nfwrite worker thread;
-    _Atomic int terminate;        // signal to terminate
-
+    fileHeaderV2_t *file_header;   // file header
+    int fd;                        // associated file descriptor
+    int compat16;                  // underlying file is compat16
+    pthread_t worker[MAXWORKERS];  // nfread/nfwrite worker thread;
+    _Atomic int terminate;         // signal to terminate
+    pthread_mutex_t wlock;         // writer lock
 #define FILE_IS_COMPAT16(n) (n->compat16)
 #define NUM_BUFFS 2
     size_t buff_size;
@@ -201,12 +206,12 @@ typedef struct nffile_s {
     dataBlock_t *block_header;  // buffer ptr
     void *buff_ptr;             // pointer into buffer for read/write blocks/records
 
-    queue_t *processQueue;  // blocks ready to be processed
-    queue_t *blockQueue;    // empty blocks
+    queue_t *processQueue;  // blocks ready to be processed. Connects consumer/producer threads
 
     stat_record_t *stat_record;  // flow stat record
     char *ident;                 // source identifier
     char *fileName;              // file name
+    uint16_t compression_level;  // compression level, if available.
 } nffile_t;
 
 #define FILE_IDENT(n) ((n)->ident)
@@ -241,7 +246,11 @@ typedef struct record_header_s {
  * for the detailed description of the record definition see nfx.h
  */
 
-int Init_nffile(queue_t *fileList);
+int Init_nffile(int workers, queue_t *fileList);
+
+int ParseCompression(char *arg);
+
+unsigned ReportBlocks(void);
 
 void SumStatRecords(stat_record_t *s1, stat_record_t *s2);
 
@@ -257,7 +266,7 @@ void PrintStat(stat_record_t *s, char *ident);
 
 void PrintGNUplotSumStat(nffile_t *nffile);
 
-int QueryFile(char *filename);
+int QueryFile(char *filename, int verbose);
 
 int GetStatRecord(char *filename, stat_record_t *stat_record);
 
