@@ -219,6 +219,10 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_postNATDestinationIPv4Address, SIZExlateDst4Addr, NumberCopy, EXnselXlateIPv4ID, OFFxlateDst4Addr, STACK_NONE, "xlate dst addr"},
     {IPFIX_postNAPTSourceTransportPort, SIZExlateSrcPort, NumberCopy, EXnselXlatePortID, OFFxlateSrcPort, STACK_NONE, "xlate src port"},
     {IPFIX_postNAPTDestinationTransportPort, SIZExlateDstPort, NumberCopy, EXnselXlatePortID, OFFxlateDstPort, STACK_NONE, "xlate dst port"},
+    // inline-monitoring inmon
+    {IPFIX_dataLinkFrameSize, SIZEframeSize, NumberCopy, EXinmonMetaID, OFFframeSize, STACK_NONE, "inmon frame size"},
+    {IPFIX_dataLinkFrameType, SIZElinkType, NumberCopy, EXinmonMetaID, OFFlinkType, STACK_NONE, "inmon link type"},
+    {IPFIX_dataLinkFrameSection, SIZEpacket, ByteCopy, EXinmonFrameID, OFFpacket, STACK_NONE, "inmon packet content"},
 
     // payload
     {LOCAL_inPayload, VARLENGTH, NumberCopy, EXinPayloadID, 0, STACK_NONE, "in payload"},
@@ -400,8 +404,6 @@ static int LookupElement(uint16_t type, uint32_t EnterpriseNumber) {
 }  // End of LookupElement
 
 static exporterDomain_t *getExporter(FlowSource_t *fs, uint32_t ObservationDomain) {
-#define IP_STRING_LEN 40
-    char ipstr[IP_STRING_LEN];
     exporterDomain_t **e = (exporterDomain_t **)&(fs->exporter_data);
 
     while (*e) {
@@ -411,17 +413,7 @@ static exporterDomain_t *getExporter(FlowSource_t *fs, uint32_t ObservationDomai
         e = &((*e)->next);
     }
 
-    if (fs->sa_family == AF_INET) {
-        uint32_t _ip = htonl(fs->ip.V4);
-        inet_ntop(AF_INET, &_ip, ipstr, sizeof(ipstr));
-    } else if (fs->sa_family == AF_INET6) {
-        uint64_t _ip[2];
-        _ip[0] = htonll(fs->ip.V6[0]);
-        _ip[1] = htonll(fs->ip.V6[1]);
-        inet_ntop(AF_INET6, &_ip, ipstr, sizeof(ipstr));
-    } else {
-        strncpy(ipstr, "<unknown>", IP_STRING_LEN);
-    }
+    char *ipstr = GetExporterIP(fs);
 
     // nothing found
     *e = (exporterDomain_t *)calloc(1, sizeof(exporterDomain_t));
@@ -1482,6 +1474,12 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
             if (observation->domainID == 0) observation->domainID = exporter->info.id;
         }
 
+        // XXX decode packet content
+        EXinmonFrame_t *inmonFrame = sequencer->offsetCache[EXinmonFrameID];
+        if (inmonFrame) {
+            // decode packet
+        }
+
         if (printRecord) {
             flow_record_short(stdout, recordHeaderV3);
         }
