@@ -63,11 +63,11 @@
 #include "ipfix.h"
 #include "launch.h"
 #include "metric.h"
-#include "netflow_pcapd.h"
 #include "netflow_v1.h"
 #include "netflow_v5_v7.h"
 #include "netflow_v9.h"
 #include "nfconf.h"
+#include "nfd_raw.h"
 #include "nfdump.h"
 #include "nffile.h"
 #include "nfnet.h"
@@ -146,9 +146,9 @@ static void usage(char *name) {
         "-A\t\tEnable source address spoofing for packet repeater -R.\n"
         "-s rate\tset default sampling rate (default 1)\n"
         "-x process\tlaunch process after a new file becomes available\n"
-        "-z\t\tLZO compress flows in output file.\n"
-        "-y\t\tLZ4 compress flows in output file.\n"
-        "-j\t\tBZ2 compress flows in output file.\n"
+        "-z=lzo\t\tLZO compress flows in output file.\n"
+        "-z=bz2\t\tBZIP2 compress flows in output file.\n"
+        "-z=lz4[:level]\tLZ4 compress flows in output file.\n"
         "-B bufflen\tSet socket buffer to bufflen bytes\n"
         "-e\t\tExpire data at each cycle.\n"
         "-D\t\tFork to background\n"
@@ -431,11 +431,9 @@ static void run(packet_function_t receive_packet, int socket, int pfd, int rfd,
                 }
 
                 // log stats
-                LogInfo(
-                    "Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad "
-                    "Packets: %u",
-                    fs->Ident, (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets,
-                    (unsigned long long)nffile->stat_record->numbytes, nffile->stat_record->sequence_failure, fs->bad_packets);
+                LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u, Blocks: %u", fs->Ident,
+                        (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets,
+                        (unsigned long long)nffile->stat_record->numbytes, nffile->stat_record->sequence_failure, fs->bad_packets, ReportBlocks());
 
                 // reset stats
                 fs->bad_packets = 0;
@@ -560,8 +558,8 @@ static void run(packet_function_t receive_packet, int socket, int pfd, int rfd,
             case 10:
                 Process_IPFIX(in_buff, cnt, fs);
                 break;
-            case 240:
-                Process_pcapd(in_buff, cnt, fs);
+            case NFD_PROTOCOL:
+                Process_nfd(in_buff, cnt, fs);
                 break;
             default:
                 // data error, while reading data from socket
@@ -633,7 +631,7 @@ int main(int argc, char **argv) {
     extensionList = NULL;
 
     int c;
-    while ((c = getopt(argc, argv, "46AB:b:C:d:DeEf:g:hI:i:jJ:l:m:M:n:p:P:R:s:S:t:T:u:vVw:x:X:yzZ")) != EOF) {
+    while ((c = getopt(argc, argv, "46AB:b:C:d:DeEf:g:hI:i:jJ:l:m:M:n:p:P:R:s:S:t:T:u:vVw:x:X:yz::Z")) != EOF) {
         switch (c) {
             case 'h':
                 usage(argv[0]);
@@ -840,7 +838,15 @@ int main(int argc, char **argv) {
                     LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression");
                     exit(EXIT_FAILURE);
                 }
-                compress = LZO_COMPRESSED;
+                if (optarg == NULL) {
+                    compress = LZO_COMPRESSED;
+                } else {
+                    compress = ParseCompression(optarg);
+                }
+                if (compress == -1) {
+                    LogError("Usage for option -z: set -z=lzo, -z=lz4 or -z=bz2 for valid compression formats");
+                    exit(EXIT_FAILURE);
+                }
                 break;
             case 'Z':
                 time_extension = "%Y%m%d%H%M%z";
@@ -872,6 +878,11 @@ int main(int argc, char **argv) {
         exit(EXIT_FAILURE);
     }
 
+    if (argc == 1) {
+        usage(argv[0]);
+        exit(EXIT_SUCCESS);
+    }
+
     if ((argc - optind) >= 2) {
         if (strcmp(argv[optind], "privsep") == 0) {
             if (strcmp(argv[optind + 1], "launcher") == 0) {
@@ -914,7 +925,7 @@ int main(int argc, char **argv) {
         exit(EXIT_FAILURE);
     }
 
-    if (!Init_nffile(NULL)) exit(254);
+    if (!Init_nffile(0, NULL)) exit(254);
 
     if (expire && spec_time_extension) {
         LogError("ERROR, -Z timezone extension breaks expire -e");
