@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2012-2022, Peter Haag
+ *  Copyright (c) 2012-2023, Peter Haag
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -94,7 +94,7 @@ int AddExporterInfo(exporter_info_record_t *exporter_record) {
         } else {
             // exporters not identical - move current slot
             int i;
-            // search first emty slot at the top of the list
+            // search first empty slot at the top of the list
             for (i = id + 1; i < MAX_EXPORTERS && exporter_list[i] != NULL; i++) {
                 ;
             }
@@ -278,6 +278,16 @@ int AddExporterStat(exporter_stats_record_t *stat_record) {
 
 }  // End of AddExporterStat
 
+exporter_t *GetExporterInfo(int exporterID) {
+    if (exporterID >= MAX_EXPORTERS) {
+        LogError("Corrupt exporter record in %s line %d\n", __FILE__, __LINE__);
+        return NULL;
+    }
+
+    return exporter_list[exporterID];
+
+}  // End of GetExporter
+
 void ExportExporterList(nffile_t *nffile) {
     // sysid 0 unused -> no exporter available
     int i = 1;
@@ -307,7 +317,6 @@ void PrintExporters(void) {
         return;
     }
 
-    uint32_t skipped_blocks = 0;
     int done = 0;
     int found = 0;
     while (!done) {
@@ -334,7 +343,6 @@ void PrintExporters(void) {
         }
 
         if (nffile->block_header->type != DATA_BLOCK_TYPE_2 && nffile->block_header->type != DATA_BLOCK_TYPE_3) {
-            skipped_blocks++;
             printf("Skip unknown block type: %u\n", nffile->block_header->type);
             continue;
         }
@@ -374,7 +382,11 @@ void PrintExporters(void) {
 
     printf("\n");
     int i = 1;
-    while (i < MAX_EXPORTERS && exporter_list[i] != NULL) {
+    while (i < MAX_EXPORTERS) {
+        if (exporter_list[i] == NULL) {
+            i++;
+            continue;
+        }
 #define IP_STRING_LEN 40
         char ipstr[IP_STRING_LEN];
 
