@@ -130,11 +130,11 @@ static void usage(char *name) {
         "\t\tand ordered by <order>: packets, bytes, flows, bps pps and bpp.\n"
         "-q\t\tQuiet: Do not print the header and bottom stat lines.\n"
         "-i <ident>\tChange Ident to <ident> in file given by -r.\n"
-        "-J <num>\tModify file compression: 0: uncompressed - 1: LZO - 2: BZ2 - 3: LZ4 "
+        "-J <num>\tModify file compression: 0: uncompressed - 1: LZO - 2: BZ2 - 3: LZ4 - 4: ZSTD"
         "compressed.\n"
-        "-z\t\tLZO compress flows in output file. Used in combination with -w.\n"
-        "-y\t\tLZ4 compress flows in output file. Used in combination with -w.\n"
-        "-j\t\tBZ2 compress flows in output file. Used in combination with -w.\n"
+        "-z=lzo\t\tLZO compress flows in output file.\n"
+        "-z=bz2\t\tBZIP2 compress flows in output file.\n"
+        "-z=lz4[:level]\tLZ4 compress flows in output file.\n"
         "-l <expr>\tSet limit on packets for line and packed output format.\n"
         "\t\tkey: 32 character string or 64 digit hex string starting with 0x.\n"
         "-L <expr>\tSet limit on bytes for line and packed output format.\n"
@@ -163,6 +163,7 @@ static void usage(char *name) {
         "\t\t\tmode may be extended by '6' for full IPv6 listing. e.g.long6, extended6.\n"
         "-E <file>\tPrint exporter and sampling info for collected flows.\n"
         "-v <file>\tverify netflow data file. Print version and blocks.\n"
+        "-W <num>\tNumber of working threads. Default number of cores online.\n"
         "-x <file>\tverify extension records in netflow data file.\n"
         "-X\t\tDump Filtertable and exit (debug option).\n"
         "-Z\t\tCheck filter syntax and exit.\n"
@@ -171,12 +172,6 @@ static void usage(char *name) {
         name);
 } /* usage */
 
-static inline void ClearMasterRecord(master_record_t *record) {
-    if (record->inPayload) free(record->inPayload);
-    if (record->outPayload) free(record->outPayload);
-    memset((void *)record, 0, sizeof(master_record_t));
-}  // End of ClearMasterRecord
-
 static void PrintSummary(stat_record_t *stat_record, outputParams_t *outputParams) {
     static double duration;
     uint64_t bps, pps, bpp;
@@ -552,13 +547,10 @@ static stat_record_t process_data(char *wfile, int element_stat, int flow_stat,
             }
         }
 
-        /* Stat info */
-        if (write_file) {
-            /* Copy stat info and close file */
-            memcpy((void *)nffile_w->stat_record, (void *)&stat_record, sizeof(stat_record_t));
-            CloseUpdateFile(nffile_w);
-            DisposeFile(nffile_w);
-        }  // else stdout
+        /* Copy stat info and close file */
+        memcpy((void *)nffile_w->stat_record, (void *)&stat_record, sizeof(stat_record_t));
+        CloseUpdateFile(nffile_w);
+        DisposeFile(nffile_w);
     }
 
     DisposeFile(nffile_r);
@@ -577,7 +569,7 @@ int main(int argc, char **argv) {
     char *print_order, *query_file, *geo_file, *configFile, *nameserver, *aggr_fmt;
     int ffd, element_stat, fdump;
     int flow_stat, aggregate, aggregate_mask, bidir;
-    int print_stat, gnuplot_stat, syntax_only, compress;
+    int print_stat, gnuplot_stat, syntax_only, compress, worker;
     int GuessDir, ModifyCompress;
     uint32_t limitRecords;
     char Ident[IDENTLEN];
@@ -597,6 +589,7 @@ int main(int argc, char **argv) {
     limitRecords = 0;
     skipped_blocks = 0;
     compress = NOT_COMPRESSED;
+    worker = 0;
     GuessDir = 0;
     nameserver = NULL;
 
@@ -619,7 +612,7 @@ int main(int argc, char **argv) {
 
     Ident[0] = '\0';
     int c;
-    while ((c = getopt(argc, argv, "6aA:Bbc:C:D:E:G:s:ghn:i:jf:qyzr:v:w:J:M:NImO:R:XZt:TVv:x:l:L:o:")) != EOF) {
+    while ((c = getopt(argc, argv, "6aA:Bbc:C:D:E:G:s:ghn:i:jf:qyz::r:v:w:J:M:NImO:R:XZt:TVv:W:x:l:L:o:")) != EOF) {
         switch (c) {
             case 'h':
                 usage(argv[0]);
@@ -661,7 +654,7 @@ int main(int argc, char **argv) {
                 }
                 break;
             case 'D':
-                CheckArgLen(optarg, 2);
+                CheckArgLen(optarg, 64);
                 nameserver = optarg;
                 if (!set_nameserver(nameserver)) {
                     exit(EXIT_FAILURE);
@@ -674,7 +667,7 @@ int main(int argc, char **argv) {
                 }
                 flist.single_file = strdup(optarg);
                 queue_t *fileList = SetupInputFileSequence(&flist);
-                if (!fileList || !Init_nffile(fileList)) exit(EXIT_FAILURE);
+                if (!fileList || !Init_nffile(1, fileList)) exit(EXIT_FAILURE);
                 PrintExporters();
                 exit(EXIT_SUCCESS);
             } break;
@@ -714,13 +707,21 @@ int main(int argc, char **argv) {
                     LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression");
                     exit(EXIT_FAILURE);
                 }
-                compress = LZO_COMPRESSED;
+                if (optarg == NULL) {
+                    compress = LZO_COMPRESSED;
+                } else {
+                    compress = ParseCompression(optarg);
+                }
+                if (compress == -1) {
+                    LogError("Usage for option -z: set -z=lzo, -z=lz4 or -z=bz2 for valid compression formats");
+                    exit(EXIT_FAILURE);
+                }
                 break;
             case 'c':
                 CheckArgLen(optarg, 16);
                 limitRecords = atoi(optarg);
                 if (!limitRecords) {
-                    LogError("Option -c needs a number > 0\n");
+                    LogError("Option -c needs a number > 0");
                     exit(EXIT_FAILURE);
                 }
                 break;
@@ -820,10 +821,9 @@ int main(int argc, char **argv) {
                 }
                 break;
             case 'J':
-                CheckArgLen(optarg, 8);
-                ModifyCompress = atoi(optarg);
-                if ((ModifyCompress < 0) || (ModifyCompress > 3)) {
-                    LogError("Expected -J <num>, 0: uncompressed, 1: LZO, 2: BZ2, 3: LZ4 compressed");
+                ModifyCompress = ParseCompression(optarg);
+                if (ModifyCompress < 0) {
+                    LogError("Expected -J <arg>, 0 for uncompressed, 1, LZO, 2, BZ2, 3, LZ4");
                     exit(EXIT_FAILURE);
                 }
                 break;
@@ -832,18 +832,26 @@ int main(int argc, char **argv) {
                 InitExtensionMaps(NO_EXTENSION_LIST);
                 flist.single_file = strdup(optarg);
                 queue_t *fileList = SetupInputFileSequence(&flist);
-                if (!fileList || !Init_nffile(fileList)) exit(EXIT_FAILURE);
+                if (!fileList || !Init_nffile(1, fileList)) exit(EXIT_FAILURE);
                 DumpExMaps();
                 exit(EXIT_SUCCESS);
             } break;
             case 'v':
                 CheckArgLen(optarg, MAXPATHLEN);
                 query_file = optarg;
-                if (!QueryFile(query_file))
+                if (!QueryFile(query_file, fdump))
                     exit(EXIT_FAILURE);
                 else
                     exit(EXIT_SUCCESS);
                 break;
+            case 'W':
+                CheckArgLen(optarg, 16);
+                worker = atoi(optarg);
+                if (worker <= 0 || worker > MAXWORKERS) {
+                    LogError("Number of working threads out of range 1..%d", MAXWORKERS);
+                    exit(EXIT_FAILURE);
+                }
+                break;
             case '6':  // print long IPv6 addr
                 Setv6Mode(1);
                 break;
@@ -944,33 +952,8 @@ int main(int argc, char **argv) {
         exit(EXIT_SUCCESS);
     }
 
-    if (geo_file == NULL) {
-        geo_file = ConfGetString("geodb.path");
-    }
-    if (geo_file && strcmp(geo_file, "none") == 0) {
-        geo_file = NULL;
-    }
-    if (geo_file) {
-        if (!CheckPath(geo_file, S_IFREG) || !Init_MaxMind() || !LoadMaxMind(geo_file)) {
-            LogError("Error reading geo location DB file %s", geo_file);
-            exit(EXIT_FAILURE);
-        }
-        HasGeoDB = true;
-        outputParams->hasGeoDB = true;
-    }
-    if (!HasGeoDB && Engine->geoFilter > 1) {
-        LogError("Can not filter according geo elements without a geo location DB");
-        exit(EXIT_FAILURE);
-    }
-    if (aggr_fmt) {
-        aggr_fmt = ParseAggregateMask(aggr_fmt, HasGeoDB);
-        if (!aggr_fmt) {
-            exit(EXIT_FAILURE);
-        }
-    }
-
     queue_t *fileList = SetupInputFileSequence(&flist);
-    if (!fileList || !Init_nffile(fileList)) exit(EXIT_FAILURE);
+    if (!fileList || !Init_nffile(worker, fileList)) exit(EXIT_FAILURE);
 
     // Modify compression
     if (ModifyCompress >= 0) {
@@ -1015,6 +998,32 @@ int main(int argc, char **argv) {
         exit(EXIT_SUCCESS);
     }
 
+    if (geo_file == NULL) {
+        geo_file = ConfGetString("geodb.path");
+    }
+    if (geo_file && strcmp(geo_file, "none") == 0) {
+        geo_file = NULL;
+    }
+    if (geo_file) {
+        if (!CheckPath(geo_file, S_IFREG) || !Init_MaxMind() || !LoadMaxMind(geo_file)) {
+            LogError("Error reading geo location DB file %s", geo_file);
+            exit(EXIT_FAILURE);
+        }
+        HasGeoDB = true;
+        outputParams->hasGeoDB = true;
+    }
+    if (!HasGeoDB && Engine->geoFilter > 1) {
+        LogError("Can not filter according geo elements without a geo location DB");
+        exit(EXIT_FAILURE);
+    }
+
+    if (aggr_fmt) {
+        aggr_fmt = ParseAggregateMask(aggr_fmt, HasGeoDB);
+        if (!aggr_fmt) {
+            exit(EXIT_FAILURE);
+        }
+    }
+
     if (gnuplot_stat) {
         nffile_t *nffile;
         if (!flist.single_file && !flist.multiple_files && !flist.multiple_dirs) {
