@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2021, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -448,7 +448,7 @@ static uint64_t pps_record(FlowHashRecord_t *record, int inout) {
 }  // End of pps_record
 
 static uint64_t bps_record(FlowHashRecord_t *record, int inout) {
-    uint64_t duration = record->msecLast - record->msecLast;
+    uint64_t duration = record->msecLast - record->msecFirst;
     if (duration == 0)
         return 0;
     else {
@@ -650,13 +650,9 @@ char *ParseAggregateMask(char *arg, int hasGeoDB) {
     aggregate_info.apply_netbits = 0;
     aggregate_info.has_masks = 0;
     aggregate_info.IPmask[0] = 0xffffffffffffffffLL;
-    ;
     aggregate_info.IPmask[1] = 0xffffffffffffffffLL;
-    ;
     aggregate_info.IPmask[2] = 0xffffffffffffffffLL;
-    ;
     aggregate_info.IPmask[3] = 0xffffffffffffffffLL;
-    ;
 
     // separate tokens
     char *p = strtok(arg, ",");
@@ -844,6 +840,7 @@ void InsertFlow(void *raw_record, master_record_t *flow_record) {
     record->counter[OUTPACKETS] = flow_record->out_pkts;
     record->counter[FLOWS] = flow_record->aggr_flows ? flow_record->aggr_flows : 1;
     record->inFlags = flow_record->tcp_flags;
+    record->outFlags = 0;
     FlowList.NumRecords++;
 
     record->next = NULL;
@@ -893,6 +890,7 @@ static void AddBidirFlow(void *raw_record, master_record_t *flow_record) {
         kh_key(FlowHash, k).counter[OUTPACKETS] = flow_record->out_pkts;
         kh_key(FlowHash, k).counter[FLOWS] = flow_record->aggr_flows ? flow_record->aggr_flows : 1;
         kh_key(FlowHash, k).inFlags = flow_record->tcp_flags;
+        kh_key(FlowHash, k).outFlags = 0;
 
         kh_key(FlowHash, k).msecFirst = flow_record->msecFirst;
         kh_key(FlowHash, k).msecLast = flow_record->msecLast;
@@ -950,6 +948,7 @@ static void AddBidirFlow(void *raw_record, master_record_t *flow_record) {
             kh_key(FlowHash, k).counter[OUTPACKETS] = flow_record->out_pkts;
             kh_key(FlowHash, k).counter[FLOWS] = flow_record->aggr_flows ? flow_record->aggr_flows : 1;
             kh_key(FlowHash, k).inFlags = flow_record->tcp_flags;
+            kh_key(FlowHash, k).outFlags = 0;
 
             kh_key(FlowHash, k).msecFirst = flow_record->msecFirst;
             kh_key(FlowHash, k).msecLast = flow_record->msecLast;
@@ -1017,6 +1016,7 @@ void AddFlowCache(void *raw_record, master_record_t *flow_record) {
         kh_key(FlowHash, k).counter[OUTPACKETS] = flow_record->out_pkts;
         kh_key(FlowHash, k).counter[FLOWS] = flow_record->aggr_flows ? flow_record->aggr_flows : 1;
         kh_key(FlowHash, k).inFlags = flow_record->tcp_flags;
+        kh_key(FlowHash, k).outFlags = 0;
 
         kh_key(FlowHash, k).msecFirst = flow_record->msecFirst;
         kh_key(FlowHash, k).msecLast = flow_record->msecLast;
@@ -1038,7 +1038,6 @@ static inline void PrintSortList(SortElement_t *SortList, uint32_t maxindex, out
 
     int max = maxindex;
     if (outputParams->topN && outputParams->topN < maxindex) max = outputParams->topN;
-
     for (int i = 0; i < max; i++) {
         int j;
 
@@ -1048,11 +1047,12 @@ static inline void PrintSortList(SortElement_t *SortList, uint32_t maxindex, out
             j = maxindex - 1 - i;
 
         FlowHashRecord_t *r = (FlowHashRecord_t *)(SortList[j].record);
-        recordHeaderV3_t *raw_record = (r->flowrecord);
+        recordHeaderV3_t *_raw_record = (r->flowrecord);
 
         master_record_t flow_record;
         memset((void *)&flow_record, 0, sizeof(master_record_t));
-        ExpandRecord_v3(raw_record, &flow_record);
+        ExpandRecord_v3(_raw_record, &flow_record);
+
         if (doGeoLookup) {
             LookupCountry(flow_record.V6.srcaddr, flow_record.src_geo);
             LookupCountry(flow_record.V6.dstaddr, flow_record.dst_geo);
@@ -1186,13 +1186,6 @@ static inline void ExportSortList(SortElement_t *SortList, uint32_t maxindex, nf
             flowMisc->revTcpFlags = r->outFlags;
             if (aggregate_info.apply_netbits) SetNetMaskBits(ipv4Flow, ipv6Flow, flowMisc, aggregate_info.apply_netbits);
         }
-        /*
-
-                        if ( aggr_record_mask ) {
-                                ApplyAggrMask(&flow_record, aggr_record_mask);
-                        }
-
-        */
         EXasRouting_t *asRouting = (EXasRouting_t *)extensionList[EXasRoutingID];
         if (genericFlow && needSwap) {
             SwapRawFlow(genericFlow, ipv4Flow, ipv6Flow, flowMisc, cntFlow, asRouting);
