@@ -88,9 +88,9 @@ static void usage(char *name) {
         "-s\t\tprofile subdir.\n"
         "-Z\t\tCheck filter syntax and exit.\n"
         "-S subdir\tSub directory format. see nfcapd(1) for format\n"
-        "-z\t\tLZO compress flows in output file.\n"
-        "-y\t\tLZ4 compress flows in output file.\n"
-        "-j\t\tBZ2 compress flows in output file.\n"
+        "-z=lzo\t\tLZO compress flows in output file.\n"
+        "-z=bz2\t\tBZIP2 compress flows in output file.\n"
+        "-z=lz4[:level]\tLZ4 compress flows in output file.\n"
 #ifdef HAVE_INFLUXDB
         "-i <influxurl>\tInfluxdb url for stats (example: http://localhost:8086/write?db=mydb&u=pippo&p=paperino)\n"
 #endif
@@ -99,17 +99,13 @@ static void usage(char *name) {
 } /* usage */
 
 static void process_data(profile_channel_info_t *channels, unsigned int num_channels, time_t tslot) {
-    nffile_t *nffile;
-    FilterEngine_t *engine;
-    int i, j, done, ret;
-
-    nffile = GetNextFile(NULL);
+    nffile_t *nffile = GetNextFile(NULL);
     if (!nffile) {
-        LogError("GetNextFile() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("GetNextFile() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return;
     }
     if (nffile == EMPTY_LIST) {
-        LogError("Empty file list. No files to process\n");
+        LogError("Empty file list. No files to process");
         return;
     }
 
@@ -119,24 +115,24 @@ static void process_data(profile_channel_info_t *channels, unsigned int num_chan
         (channels[j].engine)->ident = Ident;
     }
 
-    master_record_t *master_record = malloc(sizeof(master_record_t));
+    master_record_t *master_record = calloc(1, sizeof(master_record_t));
     if (!master_record) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return;
     }
 
-    done = 0;
+    int done = 0;
     while (!done) {
         // get next data block from file
-        ret = ReadBlock(nffile);
+        int ret = ReadBlock(nffile);
 
         switch (ret) {
             case NF_CORRUPT:
             case NF_ERROR:
                 if (ret == NF_CORRUPT)
-                    LogError("Skip corrupt data file '%s'\n", nffile->fileName);
+                    LogError("Skip corrupt data file '%s'", nffile->fileName);
                 else
-                    LogError("Read error in file '%s': %s\n", nffile->fileName, strerror(errno));
+                    LogError("Read error in file '%s': %s", nffile->fileName, strerror(errno));
                 // fall through - get next file in chain
             case NF_EOF: {
                 nffile_t *next = GetNextFile(nffile);
@@ -147,7 +143,7 @@ static void process_data(profile_channel_info_t *channels, unsigned int num_chan
                 if (next == NULL) {
                     done = 1;
                     continue;
-                    LogError("Unexpected end of file list\n");
+                    LogError("Unexpected end of file list");
                 }
 
                 strncpy(Ident, FILE_IDENT(nffile), IDENTLEN);
@@ -161,30 +157,30 @@ static void process_data(profile_channel_info_t *channels, unsigned int num_chan
         }
 
         if (nffile->block_header->type != DATA_BLOCK_TYPE_2 && nffile->block_header->type != DATA_BLOCK_TYPE_3) {
-            LogError("Can't process block type %u. Skip block.\n", nffile->block_header->type);
+            LogError("Can't process block type %u. Skip block", nffile->block_header->type);
             continue;
         }
 
         record_header_t *record_ptr = nffile->buff_ptr;
         uint32_t sumSize = 0;
-        for (i = 0; i < nffile->block_header->NumRecords; i++) {
+        for (int i = 0; i < nffile->block_header->NumRecords; i++) {
             if ((sumSize + record_ptr->size) > ret || (record_ptr->size < sizeof(record_header_t))) {
-                LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
+                LogError("Corrupt data file. Inconsistent block size in %s line %d", __FILE__, __LINE__);
                 exit(255);
             }
             sumSize += record_ptr->size;
 
             switch (record_ptr->type) {
                 case V3Record:
-                    memset((void *)master_record, 0, sizeof(master_record_t));
+                    ClearMasterRecord(master_record);
                     ExpandRecord_v3((recordHeaderV3_t *)record_ptr, master_record);
 
-                    for (j = 0; j < num_channels; j++) {
+                    for (int j = 0; j < num_channels; j++) {
                         int match;
 
                         // apply profile filter
                         (channels[j].engine)->nfrecord = (uint64_t *)master_record;
-                        engine = channels[j].engine;
+                        FilterEngine_t *engine = channels[j].engine;
                         match = (*engine->FilterEngine)(engine);
 
                         // if profile filter failed -> next profile
@@ -194,7 +190,6 @@ static void process_data(profile_channel_info_t *channels, unsigned int num_chan
 
                         // update statistics
                         UpdateStat(&channels[j].stat_record, master_record);
-                        if (channels[j].nffile) UpdateStat(channels[j].nffile->stat_record, master_record);
 
                         // do we need to write data to new file - shadow profiles do not have files.
                         // check if we need to flush the output buffer
@@ -209,29 +204,27 @@ static void process_data(profile_channel_info_t *channels, unsigned int num_chan
                 case ExporterInfoRecordType: {
                     int err = AddExporterInfo((exporter_info_record_t *)record_ptr);
                     if (err != 0) {
-                        int j;
-                        for (j = 0; j < num_channels; j++) {
+                        for (int j = 0; j < num_channels; j++) {
                             if (channels[j].nffile != NULL && err == 1) {
                                 // flush new exporter
                                 AppendToBuffer(channels[j].nffile, (void *)record_ptr, record_ptr->size);
                             }
                         }
                     } else {
-                        LogError("Failed to add Exporter Record\n");
+                        LogError("Failed to add Exporter Record");
                     }
                 } break;
                 case SamplerRecordType: {
                     int err = AddSamplerInfo((sampler_record_t *)record_ptr);
                     if (err != 0) {
-                        int j;
-                        for (j = 0; j < num_channels; j++) {
+                        for (int j = 0; j < num_channels; j++) {
                             if (channels[j].nffile != NULL && err == 1) {
                                 // flush new map
                                 AppendToBuffer(channels[j].nffile, (void *)record_ptr, record_ptr->size);
                             }
                         }
                     } else {
-                        LogError("Failed to add Sampler Record\n");
+                        LogError("Failed to add Sampler Record");
                     }
                 } break;
                 case NbarRecordType:
@@ -241,7 +234,7 @@ static void process_data(profile_channel_info_t *channels, unsigned int num_chan
                     // Silently skip exporter records
                     break;
                 default: {
-                    LogError("Skip unknown record type %i\n", record_ptr->type);
+                    LogError("Skip unknown record type %i", record_ptr->type);
                 }
             }
             // Advance pointer by number of bytes for netflow record
@@ -250,25 +243,35 @@ static void process_data(profile_channel_info_t *channels, unsigned int num_chan
         }  // End of for all umRecords
     }      // End of while !done
 
+    // Close input
+    CloseFile(nffile);
+    DisposeFile(nffile);
+
     // do we need to write data to new file - shadow profiles do not have files.
-    for (j = 0; j < num_channels; j++) {
+    // write all used blocks first, then close the files
+    for (int j = 0; j < num_channels; j++) {
         if (channels[j].nffile != NULL) {
             // flush output buffer
             if (channels[j].nffile->block_header->NumRecords) {
                 if (WriteBlock(channels[j].nffile) <= 0) {
-                    LogError("Failed to write output buffer to disk: '%s'", strerror(errno));
+                    LogError("Failed to flush output buffer to disk: '%s'", strerror(errno));
                 }
             }
+            *channels[j].nffile->stat_record = channels[j].stat_record;
+        }
+    }
+    for (int j = 0; j < num_channels; j++) {
+        if (channels[j].nffile != NULL) {
+            CloseUpdateFile(channels[j].nffile);
+            DisposeFile(channels[j].nffile);
+            channels[j].nffile = NULL;
         }
     }
-    CloseFile(nffile);
-    DisposeFile(nffile);
 
 }  // End of process_data
 
 static profile_param_info_t *ParseParams(char *profile_datadir) {
-    struct stat stat_buf;
-    char line[512], path[MAXPATHLEN], *p, *q, *s;
+    char line[512], path[MAXPATHLEN];
     profile_param_info_t *profile_list;
     profile_param_info_t **list = &profile_list;
 
@@ -280,7 +283,7 @@ static profile_param_info_t *ParseParams(char *profile_datadir) {
         // else we come from a continue statement with illegal data - overwrite
 
         if (!*list) {
-            LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             return NULL;
         }
 
@@ -294,19 +297,19 @@ static profile_param_info_t *ParseParams(char *profile_datadir) {
         // delete '\n' at the end of line
         // format of stdin config line:
         // <profilegroup>#<profilename>#<profiletype>#<channelname>#<channel_sourcelist>
-        p = strchr(line, '\n');
+        char *p = strchr(line, '\n');
         if (p) *p = '\0';
         LogInfo("Process line '%s'\n", line);
 
-        q = line;
+        char *q = line;
         p = strchr(q, '#');
         if (p) *p = '\0';
 
-        s = line;
+        char *s = line;
 
         // safety check: if no separator found loop to next line
         if (!p) {
-            LogError("Incomplete line - channel skipped.\n");
+            LogError("Incomplete line - channel skipped");
             continue;
         }
 
@@ -318,8 +321,10 @@ static profile_param_info_t *ParseParams(char *profile_datadir) {
 
         snprintf(path, MAXPATHLEN - 1, "%s/%s/%s", profile_datadir, s, q);
         path[MAXPATHLEN - 1] = '\0';
+
+        struct stat stat_buf;
         if (stat(path, &stat_buf) || !S_ISDIR(stat_buf.st_mode)) {
-            LogError("profile '%s' not found in group %s. Skipped.\n", q, s);
+            LogError("profile '%s' not found in group %s. Skipped", q, s);
             continue;
         }
 
@@ -328,7 +333,7 @@ static profile_param_info_t *ParseParams(char *profile_datadir) {
 
         // safety check: if no separator found loop to next line
         if (!p) {
-            LogError("Incomplete line - channel skipped.\n");
+            LogError("Incomplete line - channel skipped");
             continue;
         }
 
@@ -341,7 +346,7 @@ static profile_param_info_t *ParseParams(char *profile_datadir) {
         s = q;
         while (*s) {
             if (*s < '0' || *s > '9') {
-                LogError("Not a valid number: %s\n", q);
+                LogError("Not a valid number: %s", q);
                 s = NULL;
                 break;
             }
@@ -353,7 +358,7 @@ static profile_param_info_t *ParseParams(char *profile_datadir) {
 
         // safety check: if no separator found loop to next line
         if (!p) {
-            LogError("Incomplete line - channel skipped.\n");
+            LogError("Incomplete line - channel skipped");
             continue;
         }
 
@@ -366,14 +371,14 @@ static profile_param_info_t *ParseParams(char *profile_datadir) {
         snprintf(path, MAXPATHLEN - 1, "%s/%s/%s/%s", profile_datadir, (*list)->profilegroup, (*list)->profilename, q);
         path[MAXPATHLEN - 1] = '\0';
         if (stat(path, &stat_buf) || !S_ISDIR(stat_buf.st_mode)) {
-            LogError("channel '%s' in profile '%s' not found. Skipped.\n", q, (*list)->profilename);
+            LogError("channel '%s' in profile '%s' not found. Skipped", q, (*list)->profilename);
             continue;
         }
 
         (*list)->channelname = strdup(q);
 
         if (!p) {
-            LogError("Incomplete line - Skipped.\n");
+            LogError("Incomplete line - Skipped");
             continue;
         }
 
@@ -428,7 +433,7 @@ static profile_param_info_t *ParseParams(char *profile_datadir) {
 int main(int argc, char **argv) {
     unsigned int num_channels, compress;
     profile_param_info_t *profile_list;
-    char *ffile, *filename;
+    char *ffile, *filename, *syslog_facility;
     char *profile_datadir, *profile_statdir, *nameserver;
     int c, syntax_only, subdir_index, stdin_profile_params;
     time_t tslot;
@@ -444,16 +449,18 @@ int main(int argc, char **argv) {
     profile_list = NULL;
     nameserver = NULL;
     stdin_profile_params = 0;
+    syslog_facility = "daemon";
 
     // default file names
     ffile = "filter.txt";
-    while ((c = getopt(argc, argv, "D:HIL:p:P:hi:f:J;r:n:M:S:t:VzZ")) != EOF) {
+    while ((c = getopt(argc, argv, "D:Ip:P:hi:f:jr:L:M:S:t:Vyz::Z")) != EOF) {
         switch (c) {
             case 'h':
                 usage(argv[0]);
                 exit(0);
                 break;
             case 'D':
+                CheckArgLen(optarg, 64);
                 nameserver = optarg;
                 if (!set_nameserver(nameserver)) {
                     exit(255);
@@ -463,18 +470,22 @@ int main(int argc, char **argv) {
                 stdin_profile_params = 1;
                 break;
             case 'L':
-                if (!InitLog(0, "nfprofile", optarg, 0)) exit(255);
+                CheckArgLen(optarg, 32);
+                syslog_facility = strdup(optarg);
                 break;
             case 'Z':
                 syntax_only = 1;
                 break;
             case 'p':
+                CheckArgLen(optarg, MAXPATHLEN);
                 profile_datadir = optarg;
                 break;
             case 'P':
+                CheckArgLen(optarg, MAXPATHLEN);
                 profile_statdir = optarg;
                 break;
             case 'S':
+                CheckArgLen(optarg, 2);
                 subdir_index = atoi(optarg);
                 break;
             case 'V':
@@ -482,9 +493,11 @@ int main(int argc, char **argv) {
                 exit(0);
                 break;
             case 'f':
+                CheckArgLen(optarg, MAXPATHLEN);
                 ffile = optarg;
                 break;
             case 't':
+                CheckArgLen(optarg, 32);
                 tslot = atoi(optarg);
                 break;
             case 'M':
@@ -497,31 +510,39 @@ int main(int argc, char **argv) {
                 break;
             case 'j':
                 if (compress) {
-                    LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
+                    LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression");
                     exit(255);
                 }
                 compress = BZ2_COMPRESSED;
                 break;
             case 'y':
                 if (compress) {
-                    LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
+                    LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression");
                     exit(255);
                 }
                 compress = LZ4_COMPRESSED;
                 break;
             case 'z':
                 if (compress) {
-                    LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression\n");
-                    exit(255);
+                    LogError("Use one compression: -z for LZO, -j for BZ2 or -y for LZ4 compression");
+                    exit(EXIT_FAILURE);
+                }
+                if (optarg == NULL) {
+                    compress = LZO_COMPRESSED;
+                } else {
+                    compress = ParseCompression(optarg);
+                }
+                if (compress == -1) {
+                    LogError("Usage for option -z: set -z=lzo, -z=lz4 or -z=bz2 for valid compression formats");
+                    exit(EXIT_FAILURE);
                 }
-                compress = LZO_COMPRESSED;
                 break;
 #ifdef HAVE_INFLUXDB
             case 'i':
                 if (optarg != NULL)
                     strncpy(influxdb_url, optarg, 1024);
                 else {
-                    LogError("Missing argument for -i <influx URL>\n");
+                    LogError("Missing argument for -i <influx URL>");
                     exit(255);
                 }
                 influxdb_url[1023] = '\0';
@@ -533,12 +554,16 @@ int main(int argc, char **argv) {
         }
     }
 
+    if (!InitLog(1, argv[0], syslog_facility, 1)) {
+        exit(EXIT_FAILURE);
+    }
+
     if (subdir_index && !InitHierPath(subdir_index)) {
         exit(255);
     }
 
     if (!profile_datadir) {
-        LogError("Profile data directory required!\n");
+        LogError("Profile data directory required!");
         exit(255);
     }
 
@@ -548,7 +573,7 @@ int main(int argc, char **argv) {
 
     struct stat stat_buf;
     if (stat(profile_datadir, &stat_buf) || !S_ISDIR(stat_buf.st_mode)) {
-        LogError("'%s' not a directory\n", profile_datadir);
+        LogError("'%s' not a directory", profile_datadir);
         exit(255);
     }
 
@@ -565,13 +590,13 @@ int main(int argc, char **argv) {
     } else {
         char *p;
         if (flist.single_file == NULL) {
-            LogError("-r filename required!\n");
+            LogError("-r filename required!");
             exit(255);
         }
         p = strrchr(flist.single_file, '/');
         filename = p == NULL ? flist.single_file : ++p;
         if (strlen(filename) == 0) {
-            LogError("Filename error: zero length filename\n");
+            LogError("Filename error: zero length filename");
             exit(254);
         }
     }
@@ -585,7 +610,7 @@ int main(int argc, char **argv) {
 
     // nothing to do
     if (num_channels == 0) {
-        LogInfo("No channels to process.\n");
+        LogInfo("No channels to process");
         return 0;
     }
 
@@ -595,7 +620,7 @@ int main(int argc, char **argv) {
     }
 
     if (!flist.single_file) {
-        LogError("Input file (-r) required!\n");
+        LogError("Input file (-r) required!");
         exit(255);
     }
 
@@ -604,11 +629,13 @@ int main(int argc, char **argv) {
     }
 
     queue_t *fileList = SetupInputFileSequence(&flist);
-    if (!fileList || !Init_nffile(fileList)) exit(254);
+    if (!fileList || !Init_nffile(DEFAULTWORKERS, fileList)) exit(254);
 
     process_data(GetChannelInfoList(), num_channels, tslot);
 
-    CloseChannels(tslot, compress);
-
+    UpdateChannels(tslot);
+#if 0
+    VerifyFiles();
+#endif
     return 0;
 }
