@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2022, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -67,7 +67,7 @@ static int token_index = 0;
 
 #define BLOCK_SIZE 32
 
-static char **format_list = NULL;  // ordered list of all individual strings formating the output line
+static char **format_list = NULL;  // ordered list of all individual strings formatting the output line
 static int max_format_index = 0;
 
 static int do_tag = 0;
@@ -252,6 +252,10 @@ static void String_SrcLocation(FILE *stream, master_record_t *r);
 
 static void String_DstLocation(FILE *stream, master_record_t *r);
 
+static void String_SrcASorganisation(FILE *stream, master_record_t *r);
+
+static void String_DstASorganisation(FILE *stream, master_record_t *r);
+
 static void String_inPayload(FILE *stream, master_record_t *r);
 
 static void String_outPayload(FILE *stream, master_record_t *r);
@@ -278,6 +282,16 @@ static void String_evrfName(FILE *stream, master_record_t *r);
 
 static void String_NewLine(FILE *stream, master_record_t *r);
 
+static void String_pfIfName(FILE *stream, master_record_t *r);
+
+static void String_pfAction(FILE *stream, master_record_t *r);
+
+static void String_pfReason(FILE *stream, master_record_t *r);
+
+static void String_pfdir(FILE *stream, master_record_t *r);
+
+static void String_pfrule(FILE *stream, master_record_t *r);
+
 #ifdef NSEL
 static void String_EventTime(FILE *stream, master_record_t *r);
 
@@ -400,29 +414,37 @@ static struct format_token_list_s {
                           "                 ",
                           String_MPLSs},  // All MPLS labels
                          //
-                         {"%bps", 0, "     bps", String_bps},                          // bps - bits per second
-                         {"%pps", 0, "     pps", String_pps},                          // pps - packets per second
-                         {"%bpp", 0, "   Bpp", String_bpp},                            // bpp - Bytes per package
-                         {"%eng", 0, " engine", String_Engine},                        // Engine Type/ID
-                         {"%lbl", 0, "           label", String_Label},                // Flow Label
-                         {"%sc", 0, "SC", String_SrcCountry},                          // src IP 2 letter country code
-                         {"%dc", 0, "DC", String_DstCountry},                          // dst IP 2 letter country code
-                         {"%sloc", 0, "Src IP location info", String_SrcLocation},     // src IP geo location info
-                         {"%dloc", 0, "Src IP location info", String_DstLocation},     // src IP geo location info
-                         {"%n", 0, "", String_NewLine},                                // \n
-                         {"%ipl", 0, "", String_inPayload},                            // in payload
-                         {"%opl", 0, "", String_outPayload},                           // out payload
-                         {"%nbid", 0, "nbar ID", String_nbarID},                       // nbar ID
-                         {"%ja3", 0, "                             ja3", String_ja3},  // ja3
-                         {"%sni", 0, "sni name", String_sniName},                      // TLS sni Name
-                         {"%nbnam", 0, "nbar name", String_nbarName},                  // nbar Name
-                         {"%odid", 0, "obsDomainID", String_observationDomainID},      // observation domainID
-                         {"%opid", 0, "  obsPointID", String_observationPointID},      // observation pointID
-                         {"%vrf", 0, "  I-VRF-ID", String_ivrf},                       // ingress vrf ID - compatible
-                         {"%ivrf", 0, "  I-VRF-ID", String_ivrf},                      // ingress vrf ID
-                         {"%ivrfnam", 0, "  I-VRF-Name", String_ivrfName},             // ingress vrf name
-                         {"%evrf", 0, "  E-VRF-ID", String_evrf},                      // egress vrf ID
-                         {"%evrfnam", 0, "  E-VRF-Name", String_evrfName},             // egress vrf name
+                         {"%bps", 0, "     bps", String_bps},                            // bps - bits per second
+                         {"%pps", 0, "     pps", String_pps},                            // pps - packets per second
+                         {"%bpp", 0, "   Bpp", String_bpp},                              // bpp - Bytes per package
+                         {"%eng", 0, " engine", String_Engine},                          // Engine Type/ID
+                         {"%lbl", 0, "           label", String_Label},                  // Flow Label
+                         {"%sc", 0, "SC", String_SrcCountry},                            // src IP 2 letter country code
+                         {"%dc", 0, "DC", String_DstCountry},                            // dst IP 2 letter country code
+                         {"%sloc", 0, "Src IP location info", String_SrcLocation},       // src IP geo location info
+                         {"%dloc", 0, "Dst IP location info", String_DstLocation},       // dst IP geo location info
+                         {"%sasn", 0, "Src AS organisation", String_SrcASorganisation},  // src IP AS organistaion string
+                         {"%dasn", 0, "Dst AS organisation", String_DstASorganisation},  // dst IP AS organisation string
+                         {"%n", 0, "", String_NewLine},                                  // \n
+                         {"%ipl", 0, "", String_inPayload},                              // in payload
+                         {"%opl", 0, "", String_outPayload},                             // out payload
+                         {"%nbid", 0, "nbar ID", String_nbarID},                         // nbar ID
+                         {"%ja3", 0, "                             ja3", String_ja3},    // ja3
+                         {"%sni", 0, "sni name", String_sniName},                        // TLS sni Name
+                         {"%nbnam", 0, "nbar name", String_nbarName},                    // nbar Name
+                         {"%odid", 0, "obsDomainID", String_observationDomainID},        // observation domainID
+                         {"%opid", 0, "  obsPointID", String_observationPointID},        // observation pointID
+                         {"%vrf", 0, "  I-VRF-ID", String_ivrf},                         // ingress vrf ID - compatible
+                         {"%ivrf", 0, "  I-VRF-ID", String_ivrf},                        // ingress vrf ID
+                         {"%ivrfnam", 0, "  I-VRF-Name", String_ivrfName},               // ingress vrf name
+                         {"%evrf", 0, "  E-VRF-ID", String_evrf},                        // egress vrf ID
+                         {"%evrfnam", 0, "  E-VRF-Name", String_evrfName},               // egress vrf name
+
+                         {"%pfifn", 0, "interface", String_pfIfName},  // pflog ifname
+                         {"%pfact", 0, "action", String_pfAction},     // pflog action
+                         {"%pfrea", 0, "reason", String_pfReason},     // pflog reason
+                         {"%pfdir", 0, "dir", String_pfdir},           // pflog direction
+                         {"%pfrule", 0, "rule", String_pfrule},        // pflog rule
 
 #ifdef NSEL
                          // NSEL specifics
@@ -477,6 +499,16 @@ void Setv6Mode(int mode) { long_v6 += mode; }
 
 int Getv6Mode(void) { return long_v6; }
 
+static void ListOutputFormats(void) {
+    printf("Available format elements:");
+    for (int i = 0; format_token_list[i].token != NULL; i++) {
+        if ((i & 0xf) == 0) printf("\n");
+        printf("%s ", format_token_list[i].token);
+    }
+    printf("- See also nfdump(1)\n");
+
+}  // End of ListOutputFormats
+
 void fmt_record(FILE *stream, void *record, int tag) {
     master_record_t *r = (master_record_t *)record;
 
@@ -525,7 +557,7 @@ static void InitFormatParser(void) {
     format_list = (char **)calloc(1, max_format_index * sizeof(char *));
     token_list = (struct token_list_s *)calloc(1, max_token_index * sizeof(struct token_list_s));
     if (!format_list || !token_list) {
-        fprintf(stderr, "Memory allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         exit(255);
     }
 
@@ -536,7 +568,7 @@ static void AddToken(int index, char *s) {
         max_token_index += BLOCK_SIZE;
         token_list = (struct token_list_s *)realloc(token_list, max_token_index * sizeof(struct token_list_s));
         if (!token_list) {
-            fprintf(stderr, "Memory allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             exit(255);
         }
     }
@@ -552,6 +584,9 @@ static void AddToken(int index, char *s) {
 
 }  // End of AddToken
 
+/*
+ * expand predefined print format into given format, such as -o fmt "%line %ipl"
+ */
 static char *RecursiveReplace(char *format, printmap_t *printmap) {
     int i = 0;
 
@@ -567,7 +602,7 @@ static char *RecursiveReplace(char *format, printmap_t *printmap) {
                     int newlen = strlen(format) + strlen(printmap[i].Format);
                     r = malloc(newlen);
                     if (!r) {
-                        LogError("malloc() allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+                        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
                         exit(255);
                     }
                     s[0] = '\0';
@@ -595,12 +630,11 @@ int ParseOutputFormat(char *format, int plain_numbers, printmap_t *printmap) {
 
     s = strdup(format);
     if (!s) {
-        fprintf(stderr, "Memory allocation error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
-        exit(255);
+        LogError("malloc() allocation error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+        return 0;
     }
     s = RecursiveReplace(s, printmap);
     c = s;
-
     h = header_string;
     *h = '\0';
     while (*c) {
@@ -611,7 +645,7 @@ int ParseOutputFormat(char *format, int plain_numbers, printmap_t *printmap) {
                 int len = strlen(format_token_list[i].token);
 
                 // a token is separated by either a space, another token, or end of string
-                if (remaining >= len && !isalpha((int)c[len])) {
+                if (remaining >= len && !isalnum((int)c[len])) {
                     // separator found a expected position
                     char p = c[len];  // save separator;
                     c[len] = '\0';
@@ -632,8 +666,9 @@ int ParseOutputFormat(char *format, int plain_numbers, printmap_t *printmap) {
                 i++;
             }
             if (format_token_list[i].token == NULL) {
-                fprintf(stderr, "Output format parse error at: %s\n", c);
+                LogError("Output format parse error at: %s", c);
                 free(s);
+                ListOutputFormats();
                 return 0;
             }
         } else {  // it's a static string
@@ -672,7 +707,7 @@ static char *ICMP_Port_decode(master_record_t *r) {
     static char icmp_string[ICMPSTRLEN];
 
     if (r->proto == IPPROTO_ICMP || r->proto == IPPROTO_ICMPV6) {  // ICMP
-        snprintf(icmp_string, ICMPSTRLEN - 1, "%u.%u", r->icmp_type, r->icmp_code);
+        snprintf(icmp_string, ICMPSTRLEN - 1, "%u.%u", r->icmpType, r->icmpCode);
     } else {  // dst port
         snprintf(icmp_string, ICMPSTRLEN - 1, "%u", r->dstPort);
     }
@@ -762,7 +797,7 @@ static void String_FirstSeenRaw(FILE *stream, master_record_t *r) {
 
 static void String_LastSeenRaw(FILE *stream, master_record_t *r) {
     /* snprintf does write \0, and the max is INCL the terminating \0 */
-    fprintf(stream, "%llu.%03llu", r->msecLast / 1000LL, r->msecLast % 100LL);
+    fprintf(stream, "%llu.%03llu", r->msecLast / 1000LL, r->msecLast % 1000LL);
 
 }  // End of String_LastSeenRaw
 
@@ -1299,19 +1334,13 @@ static void String_SrcPort(FILE *stream, master_record_t *r) { fprintf(stream, "
 static void String_DstPort(FILE *stream, master_record_t *r) { fprintf(stream, "%6s", ICMP_Port_decode(r)); }  // End of String_DstPort
 
 static void String_ICMP_type(FILE *stream, master_record_t *r) {
-    uint8_t type;
-
-    type = (r->proto == IPPROTO_ICMP || r->proto == IPPROTO_ICMPV6) ? r->icmp_type : 0;
-    fprintf(stream, "%6u", type);
-
+    // Force printing type regardless of protocol
+    fprintf(stream, "%6u", r->icmpType);
 }  // End of String_ICMP_type
 
 static void String_ICMP_code(FILE *stream, master_record_t *r) {
-    uint8_t code;
-
-    code = (r->proto == IPPROTO_ICMP || r->proto == IPPROTO_ICMPV6) ? r->icmp_code : 0;
-    fprintf(stream, "%6u", code);
-
+    // Force printing code regardless of protocol
+    fprintf(stream, "%6u", r->icmpCode);
 }  // End of String_ICMP_code
 
 static void String_SrcAS(FILE *stream, master_record_t *r) {
@@ -1619,6 +1648,14 @@ static void String_DstLocation(FILE *stream, master_record_t *r) {
 
 }  // End of String_DstLocation
 
+static void String_SrcASorganisation(FILE *stream, master_record_t *r) {
+    fprintf(stream, "%s", LookupASorg(r->V6.srcaddr));
+}  // End of String_SrcLocation
+
+static void String_DstASorganisation(FILE *stream, master_record_t *r) {
+    fprintf(stream, "%s", LookupASorg(r->V6.dstaddr));
+}  // End of String_DstLocation
+
 static void String_ivrf(FILE *stream, master_record_t *r) { fprintf(stream, "%10u", r->ingressVrf); }  // End of String_ivrf
 
 static void String_evrf(FILE *stream, master_record_t *r) { fprintf(stream, "%10u", r->egressVrf); }  // End of String_evrf
@@ -1633,6 +1670,31 @@ static void String_evrfName(FILE *stream, master_record_t *r) {
     fprintf(stream, "%s", GetVrfName(r->egressVrf, vrfName, sizeof(vrfName)));
 }  // End of String_evrfName
 
+static void String_pfIfName(FILE *stream, master_record_t *r) {
+    //
+    fprintf(stream, "%9s", r->pfIfName);
+}  // End of String_pfIfName
+
+static void String_pfAction(FILE *stream, master_record_t *r) {
+    //
+    fprintf(stream, "%6s", pfAction(r->pfAction));
+}  // End of String_pfAction
+
+static void String_pfReason(FILE *stream, master_record_t *r) {
+    //
+    fprintf(stream, "%6s", pfReason(r->pfReason));
+}  // End of String_pfReason
+
+static void String_pfdir(FILE *stream, master_record_t *r) {
+    //
+    fprintf(stream, "%3s", r->pfDir ? "in" : "out");
+}  // End of String_pfdir
+
+static void String_pfrule(FILE *stream, master_record_t *r) {
+    //
+    fprintf(stream, "%4u", r->pfRulenr);
+}  // End of String_pfrule
+
 #ifdef NSEL
 static void String_nfc(FILE *stream, master_record_t *r) { fprintf(stream, "%10u", r->connID); }  // End of String_nfc
 
