@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2021, Peter Haag
+ *  Copyright (c) 2009-2023, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -76,7 +76,7 @@ typedef struct value64_s {
 static char *CurrentIdent;
 static FilterEngine_t *Engine;
 
-/* exported fuctions */
+/* exported functions */
 static int check_filter_block(char *filter, master_record_t *flow_record, int expect);
 
 static void check_offset(char *text, pointer_addr_t offset, pointer_addr_t expect);
@@ -217,35 +217,32 @@ int main(int argc, char **argv) {
 
     flow_record.srcPort = 0xaaaa;
     flow_record.proto = IPPROTO_ICMP;
-    flow_record.dstPort = 0xaffa;  // -> icmp type 175, code 250
-    flow_record.icmp = flow_record.dstPort;
-    flow_record.dstPort = 0xbbbb;
+    // -> icmp type 175, code 250
+    flow_record.dstPort = 175 * 256 + 250;
     ret = check_filter_block("icmp-type 175", &flow_record, 1);
     ret = check_filter_block("icmp-type 176", &flow_record, 0);
     ret = check_filter_block("icmp-code 250", &flow_record, 1);
     ret = check_filter_block("icmp-code 251", &flow_record, 0);
 
-    if (flow_record.icmp_type != 175 && flow_record.icmp_code != 250) {
+    if (flow_record.icmpType != 175 && flow_record.icmpCode != 250) {
         printf("**** FAILED **** ICMP type check failed!\n");
-        printf("ICMP type: %u, code: %u\n", flow_record.icmp_type, flow_record.icmp_code);
+        printf("ICMP type: %u, code: %u\n", flow_record.icmpType, flow_record.icmpCode);
         exit(255);
     }
 
     flow_record.dstPort = 3 << 8;  // -> icmp type 3
-    flow_record.icmp = flow_record.dstPort;
-    if (flow_record.icmp_type != 3) {
+    if (flow_record.icmpType != 3) {
         printf("**** FAILED **** ICMP type check failed!\n");
-        printf("ICMP type: %u, code: %u\n", flow_record.icmp_type, flow_record.icmp_code);
+        printf("ICMP type: %u, code: %u\n", flow_record.icmpType, flow_record.icmpCode);
         exit(255);
     }
     ret = check_filter_block("icmp-type 3", &flow_record, 1);
     ret = check_filter_block("icmp-type 4", &flow_record, 0);
 
     flow_record.dstPort = 8;  // -> icmp code 8
-    flow_record.icmp = flow_record.dstPort;
-    if (flow_record.icmp_code != 8) {
+    if (flow_record.icmpCode != 8) {
         printf("**** FAILED **** ICMP code check failed!\n");
-        printf("ICMP type: %u, code: %u\n", flow_record.icmp_type, flow_record.icmp_code);
+        printf("ICMP type: %u, code: %u\n", flow_record.icmpType, flow_record.icmpCode);
         exit(255);
     }
     ret = check_filter_block("icmp-code 8", &flow_record, 1);
@@ -546,6 +543,11 @@ int main(int argc, char **argv) {
     ret = check_filter_block("engine-id 6", &flow_record, 1);
     ret = check_filter_block("engine-id 7", &flow_record, 0);
 
+    flow_record.exporter_sysid = 5;
+    ret = check_filter_block("exporter 5", &flow_record, 1);
+    ret = check_filter_block("exporter 6", &flow_record, 0);
+    ret = check_filter_block("exporter < 6", &flow_record, 1);
+
     flow_record.proto = IPPROTO_TCP;
     flow_record.tcp_flags = 1;
     ret = check_filter_block("flags F", &flow_record, 1);
@@ -1000,6 +1002,41 @@ int main(int argc, char **argv) {
     ret = check_filter_block("egress vrf 0xBBBB", &flow_record, 1);
     ret = check_filter_block("egress vrf 0xAAAA", &flow_record, 0);
 
+    flow_record.pfAction = 1;
+    ret = check_filter_block("pf action block", &flow_record, 1);
+    ret = check_filter_block("pf action pass", &flow_record, 0);
+    flow_record.pfAction = 0;
+    ret = check_filter_block("pf action pass", &flow_record, 1);
+    flow_record.pfAction = 4;
+    ret = check_filter_block("pf action pass", &flow_record, 0);
+    ret = check_filter_block("pf action nat", &flow_record, 1);
+
+    flow_record.pfReason = 0;
+    ret = check_filter_block("pf reason match", &flow_record, 1);
+    ret = check_filter_block("pf reason short", &flow_record, 0);
+    flow_record.pfReason = 3;
+    ret = check_filter_block("pf reason short", &flow_record, 1);
+
+    flow_record.pfRulenr = 22;
+    ret = check_filter_block("pf rule 22", &flow_record, 1);
+    ret = check_filter_block("pf rule 23", &flow_record, 0);
+    flow_record.pfRulenr = 23;
+    ret = check_filter_block("pf rule 23", &flow_record, 1);
+
+    flow_record.pfDir = 1;
+    ret = check_filter_block("pf dir in", &flow_record, 1);
+    ret = check_filter_block("pf dir out", &flow_record, 0);
+    flow_record.pfDir = 0;
+    ret = check_filter_block("pf dir out", &flow_record, 1);
+
+    memset(flow_record.pfIfName, 0, 16);
+    memcpy(flow_record.pfIfName, "vether0", 7);
+    ret = check_filter_block("pf interface vether0", &flow_record, 1);
+    ret = check_filter_block("pf interface vmx0", &flow_record, 0);
+    memcpy(flow_record.pfIfName, "longinterface", 13);
+    ret = check_filter_block("pf interface longinterface", &flow_record, 1);
+    ret = check_filter_block("pf interface vether0", &flow_record, 0);
+
     // NSEL/ASA related tests
 #ifdef NSEL
     flow_record.event = NSEL_EVENT_IGNORE;
@@ -1024,7 +1061,7 @@ int main(int argc, char **argv) {
     flow_record.fwXevent = 1002;
     ret = check_filter_block("asa event denied egress", &flow_record, 1);
     flow_record.fwXevent = 1003;
-    ret = check_filter_block("asa event denied interface", &flow_record, 1);
+    // ret = check_filter_block("asa event denied interface", &flow_record, 1);
     flow_record.fwXevent = 1004;
     ret = check_filter_block("asa event denied nosyn", &flow_record, 1);
     ret = check_filter_block("asa event denied ingress", &flow_record, 0);
